@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_matroska.c,v 1.51 2007/01/19 00:26:40 dgp85 Exp $
+ * $Id: demux_matroska.c,v 1.53 2007/03/12 16:27:21 dgp85 Exp $
  *
  * demultiplexer for matroska streams
  *
@@ -1257,7 +1257,7 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
         if (!ebml_read_uint(ebml, &elem, &val))
           return 0;
         track->default_duration = val;
-        lprintf("Default Duration: %lld\n", track->default_duration);
+        lprintf("Default Duration: %"PRIu64"\n", track->default_duration);
       }
       break;
 
@@ -1396,6 +1396,10 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       init_codec = init_codec_audio;
       
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_DTS)) {
+      lprintf("MATROSKA_CODEC_ID_A_DTS\n");
+      track->buf_type = BUF_AUDIO_DTS;
+      init_codec = init_codec_audio;
+
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_VORBIS)) {
 
       lprintf("MATROSKA_CODEC_ID_A_VORBIS\n");
