@@ -1,23 +1,24 @@
 /*
- *  Copyright (C) 2002-2003 the xine project
+ *  Copyright (C) 2002-2003,2007 the xine project
  *
  *  This file is part of xine, a free video player.
  *
- *  xine is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- * 
- *  xine is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * The xine-lib XML parser is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * The xine-lib XML parser is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- *  $Id: xmllexer.c,v 1.12 2006/06/20 00:35:08 dgp85 Exp $
+ * You should have received a copy of the GNU Library General Public
+ * License along with the Gnome Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ *  $Id: xmllexer.c,v 1.13 2007/03/04 16:19:12 hadess Exp $
  *
  */
 
@@ -27,11 +28,17 @@
 #define LOG
 */
 
+#ifdef XINE_COMPILE
 #include "xineutils.h"
+#else
+#define lprintf(...)
+#define xine_xmalloc malloc
+#endif
 #include "xmllexer.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
+#include <stdlib.h>
 
 /* private constants*/
 #define NORMAL       0  /* normal lex mode */
